import numpy as np
from pyTurbulence.syntheticTurbulence import *
from pyTurbulence.plot3D import *
from pyTurbulence.spectrum import *
import time

params = {
    "nbModes"           : 512, 
    "Mt"                : 0.23,
    "Pressure"          : 1.0,   
    "Temperature"       : 1.0,
    "k0"                : 8.0,
    "domain_size"       : (2. * np.pi, 2. * np.pi, 2. * np.pi),
    "domain_resolution" : (256,256,256),
    "seed"              : 42,
    "gamma"             : 1.4,
    "case"              : 1,
    "Spectrum"          : "PassotPouquet"
}

# read the params dictionnary 
nbModes = params["nbModes"]
Mt = params["Mt"]
mean_pressure = params["Pressure"]
mean_temperature = params["Temperature"]
mean_density = mean_pressure / mean_temperature
gamma = params["gamma"]
celerity = np.sqrt(gamma * mean_pressure / mean_density)
Ek = 0.5*(Mt*celerity)**2
urms = np.sqrt(2. * Ek / 3.)
k0 = params["k0"]
domain_size = params["domain_size"]
domain_resolution = params["domain_resolution"]
xi = gamma*Mt**2
# xi = 0.0
spectrum = params["Spectrum"]

# Generate the solenoidal velocity field
start = time.time()
u,v,w,wmax = compute_solenoidal_velocities(spectrum, nbModes, urms, k0, domain_size, domain_resolution, seed=42)
TKE = np.mean(0.5*(u.reshape(-1)**2 + v.reshape(-1)**2 + w.reshape(-1)**2))
end = time.time()
print(f"done computing the solenoidal velocity field with TKE = {TKE}, expected TKE = {Ek} in {end-start} seconds")

# Generate the incompressible pressure fluctuations
start = time.time()
incompressible_pressure_fluctuations = compute_solenoidal_pressure(u,v,w, domain_resolution, domain_size)
end = time.time()
print(f"done computing the incompressible pressure fluctuations in {end-start} seconds")

# Compute the thermodynamic fields 
start = time.time()
density, pressure, temperature = compute_thermodynamic_fields(mean_density, 
                                                              mean_pressure, 
                                                              mean_temperature, 
                                                              incompressible_pressure_fluctuations, 
                                                              gamma, Mt, params["case"])
end = time.time()
print(f"done computing the thermodynamic fields in {end-start} seconds")

# Plot the spectrum
start = time.time()
knyquist, wave_numbers, tke_spectrum = compute_tke_spectrum(u, v, w, 
                                                            domain_size[0], 
                                                            domain_size[1], 
                                                            domain_size[2])
plot_spectrum(wmax, wave_numbers, tke_spectrum)
end = time.time()
print(f"done plotting the spectrum in {end-start} seconds")

# Generate the visualisation 
# plot3D(u, domain_size, domain_resolution, "U")
# plot3D(pressure, domain_size, domain_resolution, "Pressure")

# Save the data
start = time.time()
u.tofile("u.dat")
v.tofile("v.dat")
w.tofile("w.dat")
pressure.tofile("p.dat")
temperature.tofile("T.dat")
end = time.time()
print(f"done saving the data in {end-start} seconds")
